@@ -1,29 +1,31 @@
-from typing import Tuple
+from __future__ import annotations
+
+from fractions import Fraction
 
 import av
 import numpy as np
 from numpy.typing import NDArray
 from PIL import ImageChops, ImageFilter, ImageOps
 
-from auto_editor.utils.progressbar import ProgressBar
+from auto_editor.utils.bar import Bar
 
 
-def new_size(size: Tuple[int, int], width: int) -> Tuple[int, int]:
+def new_size(size: tuple[int, int], width: int) -> tuple[int, int]:
     h, w = size
     return width, int(h * (width / w))
 
 
 def motion_detection(
-    path: str, fps: float, progress: ProgressBar, width: int, blur: int
+    path: str, track: int, tb: Fraction, bar: Bar, width: int, blur: int
 ) -> NDArray[np.float_]:
     container = av.open(path, "r")
 
-    stream = container.streams.video[0]
+    stream = container.streams.video[track]
     stream.thread_type = "AUTO"
 
     inaccurate_dur = int(stream.duration * stream.time_base * stream.rate)
 
-    progress.start(inaccurate_dur, "Analyzing motion")
+    bar.start(inaccurate_dur, "Analyzing motion")
 
     prev_image = None
     image = None
@@ -33,14 +35,10 @@ def motion_detection(
     threshold_list = np.zeros((1024), dtype=np.float_)
 
     for frame in container.decode(stream):
-        if image is None:
-            prev_image = None
-        else:
-            prev_image = image
+        prev_image = image
 
-        index = int(frame.time * fps)
-
-        progress.tick(index)
+        index = int(frame.time * tb)
+        bar.tick(index)
 
         if index > len(threshold_list) - 1:
             threshold_list = np.concatenate(
@@ -64,5 +62,5 @@ def motion_detection(
 
             threshold_list[index] = count / total_pixels
 
-    progress.end()
+    bar.end()
     return threshold_list[:index]
