@@ -1,13 +1,15 @@
+from __future__ import annotations
+
 import json
 import os.path
 import subprocess
 import sys
 from dataclasses import dataclass
 from fractions import Fraction
-from platform import system
+from platform import machine, system
 from re import search
 from subprocess import PIPE, Popen
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any
 
 from auto_editor.utils.func import get_stdout
 from auto_editor.utils.log import Log
@@ -19,47 +21,42 @@ SUB_EXTS = {"mov_text": "srt", "ass": "ass", "webvtt": "vtt"}
 class FFmpeg:
     __slots__ = ("debug", "path", "version")
 
-    @staticmethod
-    def _set_ff_path(ff_location: Optional[str], my_ffmpeg: bool) -> str:
-        if ff_location is not None:
-            return ff_location
-        if my_ffmpeg or system() not in ("Windows", "Darwin"):
-            return "ffmpeg"
-
-        # Assumming accessing __file__ is somewhat expensive.
-        program = "ffmpeg.exe" if system() == "Windows" else "ffmpeg"
-        dirpath = os.path.dirname(os.path.realpath(__file__))
-        file_path = os.path.join(dirpath, "ffmpeg", system(), program)
-
-        # May not exist because auto-editor doesn't bundle every architecture.
-        if os.path.isfile(file_path):
-            return file_path
-
-        return "ffmpeg"
-
     def __init__(
         self,
-        ff_location: Optional[str] = None,
+        ff_location: str | None = None,
         my_ffmpeg: bool = False,
         debug: bool = False,
     ) -> None:
+        def _set_ff_path(ff_location: str | None, my_ffmpeg: bool) -> str:
+            if ff_location is not None:
+                return ff_location
+            if my_ffmpeg:
+                return "ffmpeg"
+
+            try:
+                import ae_ffmpeg
+
+                return ae_ffmpeg.get_path()
+            except ImportError:
+                return "ffmpeg"
 
         self.debug = debug
-        self.path = self._set_ff_path(ff_location, my_ffmpeg)
+        self.path = _set_ff_path(ff_location, my_ffmpeg)
         try:
             _version = get_stdout([self.path, "-version"]).split("\n")[0]
             _version = _version.replace("ffmpeg version", "").strip()
             self.version = _version.split(" ")[0]
         except FileNotFoundError:
             if system() == "Darwin":
+                if machine() == "arm64":
+                    Log().error("No ffmpeg found, download via homebrew.")
                 Log().error(
-                    "No ffmpeg found, download via homebrew or restore the "
-                    "included binary."
+                    "No ffmpeg found, download via homebrew or install ae-ffmpeg."
                 )
             if system() == "Windows":
                 Log().error(
                     "No ffmpeg found, download ffmpeg with your favorite package "
-                    "manager (ex chocolatey), or restore the included binary."
+                    "manager (ex chocolatey), or install ae-ffmpeg."
                 )
 
             Log().error("ffmpeg must be installed and on PATH.")
@@ -68,11 +65,11 @@ class FFmpeg:
         if self.debug:
             sys.stderr.write(f"FFmpeg: {message}\n")
 
-    def print_cmd(self, cmd: List[str]) -> None:
+    def print_cmd(self, cmd: list[str]) -> None:
         if self.debug:
             sys.stderr.write(f"FFmpeg run: {' '.join(cmd)}\n")
 
-    def run(self, cmd: List[str]) -> None:
+    def run(self, cmd: list[str]) -> None:
         cmd = [self.path, "-y", "-hide_banner"] + cmd
         if not self.debug:
             cmd.extend(["-nostats", "-loglevel", "error"])
@@ -81,12 +78,12 @@ class FFmpeg:
 
     def run_check_errors(
         self,
-        cmd: List[str],
+        cmd: list[str],
         log: Log,
         show_out: bool = False,
-        path: Optional[str] = None,
+        path: str | None = None,
     ) -> None:
-        def _run(cmd: List[str]) -> str:
+        def _run(cmd: list[str]) -> str:
             process = self.Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
 
             _, stderr = process.communicate()
@@ -119,8 +116,7 @@ class FFmpeg:
             print(f"stderr: {output}")
 
         for item in error_list:
-            check = search(item, output)
-            if check:
+            if check := search(item, output):
                 log.error(check.group())
 
         if path is not None and not os.path.isfile(path):
@@ -128,12 +124,12 @@ class FFmpeg:
         elif show_out and not self.debug:
             print(f"stderr: {output}")
 
-    def Popen(self, cmd: List[str], stdin=None, stdout=PIPE, stderr=None) -> Popen:
+    def Popen(self, cmd: list[str], stdin=None, stdout=PIPE, stderr=None) -> Popen:
         cmd = [self.path] + cmd
         self.print_cmd(cmd)
         return Popen(cmd, stdin=stdin, stdout=stdout, stderr=stderr)
 
-    def pipe(self, cmd: List[str]) -> str:
+    def pipe(self, cmd: list[str]) -> str:
         cmd = [self.path, "-y"] + cmd
 
         self.print_cmd(cmd)
@@ -147,30 +143,33 @@ class VideoStream:
     width: int
     height: int
     codec: str
-    fps: float
+    fps: Fraction
+    duration: str | None
+    sar: str | None
     time_base: Fraction
     pix_fmt: str
-    color_range: Optional[str]
-    color_space: Optional[str]
-    color_primaries: Optional[str]
-    color_transfer: Optional[str]
-    bitrate: Optional[str]
-    lang: Optional[str]
+    color_range: str | None
+    color_space: str | None
+    color_primaries: str | None
+    color_transfer: str | None
+    bitrate: str | None
+    lang: str | None
 
 
 @dataclass
 class AudioStream:
     codec: str
     samplerate: int
-    bitrate: Optional[str]
-    lang: Optional[str]
+    duration: str | None
+    bitrate: str | None
+    lang: str | None
 
 
 @dataclass
 class SubtitleStream:
     codec: str
     ext: str
-    lang: Optional[str]
+    lang: str | None
 
 
 class FileInfo:
@@ -182,23 +181,22 @@ class FileInfo:
         "name",
         "ext",
         "bitrate",
+        "duration",
         "description",
         "videos",
         "audios",
         "subtitles",
     )
 
-    def get_res(self) -> Tuple[int, int]:
+    def get_res(self) -> tuple[int, int]:
         if len(self.videos) > 0:
             return self.videos[0].width, self.videos[0].height
         return 1920, 1080
 
-    def get_fps(self) -> float:
-        fps = None
+    def get_fps(self) -> Fraction:
         if len(self.videos) > 0:
-            fps = self.videos[0].fps
-
-        return 30 if fps is None else fps
+            return self.videos[0].fps
+        return Fraction(30)
 
     def get_samplerate(self) -> int:
         if len(self.audios) > 0:
@@ -212,10 +210,11 @@ class FileInfo:
         self.dirname = os.path.dirname(os.path.abspath(path))
         self.name, self.ext = os.path.splitext(path)
 
-        self.videos: List[VideoStream] = []
-        self.audios: List[AudioStream] = []
-        self.subtitles: List[SubtitleStream] = []
+        self.videos: list[VideoStream] = []
+        self.audios: list[AudioStream] = []
+        self.subtitles: list[SubtitleStream] = []
         self.description = None
+        self.duration = ""
 
         _dir = os.path.dirname(ffmpeg.path)
         _ext = os.path.splitext(ffmpeg.path)[1]
@@ -237,7 +236,7 @@ class FileInfo:
         except FileNotFoundError:
             log.error(f"Could not find: {ffprobe}")
 
-        def get_attr(name: str, dic: Dict[str, Any], default=-1) -> str:
+        def get_attr(name: str, dic: dict[str, Any], default=-1) -> str:
             if name in dic:
                 if isinstance(dic[name], str):
                     return dic[name]
@@ -259,7 +258,7 @@ class FileInfo:
         except Exception as e:
             log.error(f"{path}: Could not read ffprobe JSON: {e}")
 
-        self.bitrate: Optional[str] = None
+        self.bitrate: str | None = None
         if "bit_rate" in json_info["format"]:
             self.bitrate = json_info["format"]["bit_rate"]
         if (
@@ -268,6 +267,9 @@ class FileInfo:
         ):
             self.description = json_info["format"]["tags"]["description"]
 
+        if "duration" in json_info["format"]:
+            self.duration = json_info["format"]["duration"]
+
         for stream in json_info["streams"]:
             lang = None
             br = None
@@ -283,32 +285,34 @@ class FileInfo:
 
             if codec_type == "video":
                 pix_fmt = get_attr("pix_fmt", stream)
+                vduration = get_attr("duration", stream, default=None)
                 color_range = get_attr("color_range", stream, default=None)
                 color_space = get_attr("color_space", stream, default=None)
                 color_primaries = get_attr("color_primaries", stream, default=None)
                 color_transfer = get_attr("color_transfer", stream, default=None)
-                fps_str = get_attr("avg_frame_rate", stream)
+                sar = get_attr("sample_aspect_ratio", stream, default=None)
+                fps_str = get_attr("r_frame_rate", stream)
                 time_base_str = get_attr("time_base", stream)
 
                 try:
-                    fps = float(Fraction(fps_str))
+                    fps = Fraction(fps_str)
                 except ZeroDivisionError:
-                    fps = 0
+                    fps = Fraction(0)
                 except ValueError:
-                    log.error(f"Could not convert fps '{fps_str}' to float")
+                    log.error(f"Could not convert fps '{fps_str}' to Fraction.")
 
                 if fps < 1:
                     if codec in IMG_CODECS:
-                        fps = 25
-                    else:
-                        log.error("fps cannot be less than 1.")
+                        fps = Fraction(25)
+                    elif fps == 0:
+                        fps = Fraction(30)
 
                 try:
                     time_base = Fraction(time_base_str)
                 except (ValueError, ZeroDivisionError):
                     if codec not in IMG_CODECS:
                         log.error(
-                            f"Could not convert time_base '{time_base_str}' to Fraction"
+                            f"Could not convert time_base '{time_base_str}' to Fraction."
                         )
                     time_base = Fraction(0, 1)
 
@@ -318,6 +322,8 @@ class FileInfo:
                         stream["height"],
                         codec,
                         fps,
+                        vduration,
+                        sar,
                         time_base,
                         pix_fmt,
                         color_range,
@@ -330,7 +336,8 @@ class FileInfo:
                 )
             if codec_type == "audio":
                 sr = int(stream["sample_rate"])
-                self.audios.append(AudioStream(codec, sr, br, lang))
+                adur = get_attr("duration", stream, default=None)
+                self.audios.append(AudioStream(codec, sr, adur, br, lang))
             if codec_type == "subtitle":
                 ext = SUB_EXTS.get(codec, "vtt")
                 self.subtitles.append(SubtitleStream(codec, ext, lang))
