@@ -1,20 +1,23 @@
+from __future__ import annotations
+
 import os.path
-import subprocess
 from dataclasses import dataclass
 from math import ceil
-from typing import Dict, List, Tuple, Union
+from subprocess import DEVNULL, PIPE
+from typing import Dict, Tuple, Union
 
 import av
 from PIL import Image, ImageChops, ImageDraw, ImageFont, ImageOps
 
 from auto_editor.ffwrapper import FFmpeg
+from auto_editor.make_layers import Visual
 from auto_editor.objects import EllipseObj, ImageObj, RectangleObj, TextObj, VideoObj
-from auto_editor.output import get_vcodec, video_quality
-from auto_editor.timeline import Timeline, Visual
+from auto_editor.output import video_quality
+from auto_editor.timeline import Timeline
+from auto_editor.utils.bar import Bar
 from auto_editor.utils.container import Container
 from auto_editor.utils.encoder import encoders
 from auto_editor.utils.log import Log
-from auto_editor.utils.progressbar import ProgressBar
 from auto_editor.utils.types import Args
 
 av.logging.set_level(av.logging.PANIC)
@@ -52,7 +55,7 @@ allowed_pix_fmt = {
 
 def apply_anchor(
     x: int, y: int, width: int, height: int, anchor: str
-) -> Tuple[int, int]:
+) -> tuple[int, int]:
     if anchor == "ce":
         x = int((x * 2 - width) / 2)
         y = int((y * 2 - height) / 2)
@@ -69,7 +72,7 @@ def apply_anchor(
 
 def one_pos_two_pos(
     x: int, y: int, width: int, height: int, anchor: str
-) -> Tuple[int, int, int, int]:
+) -> tuple[int, int, int, int]:
     """Convert: x, y, width, height -> x1, y1, x2, y2"""
 
     if anchor == "ce":
@@ -101,18 +104,18 @@ def render_av(
     ffmpeg: FFmpeg,
     timeline: Timeline,
     args: Args,
-    progress: ProgressBar,
+    bar: Bar,
     ctr: Container,
     temp: str,
     log: Log,
-) -> Tuple[str, bool]:
+) -> tuple[str, bool]:
 
     FontCache = Dict[
         str, Tuple[Union[ImageFont.FreeTypeFont, ImageFont.ImageFont], float]
     ]
 
     font_cache: FontCache = {}
-    img_cache: Dict[str, Image.Image] = {}
+    img_cache: dict[str, Image.Image] = {}
     for layer in timeline.v:
         for vobj in layer:
             if isinstance(vobj, TextObj) and (vobj.font, vobj.size) not in font_cache:
@@ -141,27 +144,46 @@ def render_av(
     inp = timeline.inp
     cns = [av.open(inp.path, "r") for inp in timeline.inputs]
 
+    target_pix_fmt = "yuv420p"  # Reasonable default
     decoders = []
     tous = []
-    pix_fmts = []
-    for cn in cns:
-        stream = cn.streams.video[0]
-        stream.thread_type = "AUTO"
+    seek_cost = []
+    for c, cn in enumerate(cns):
+        if len(cn.streams.video) == 0:
+            decoders.append(None)
+            tous.append(0)
+            seek_cost.append(0)
+        else:
+            stream = cn.streams.video[0]
+            stream.thread_type = "AUTO"
 
-        tous.append(int(stream.time_base.denominator / stream.average_rate))
-        pix_fmts.append(stream.pix_fmt)
-        decoders.append(cn.decode(stream))
+            if args.no_seek or stream.average_rate is None:
+                sc_val = 4294967295  # 2 ** 32 - 1
+                tou = 0
+            else:
+                # Keyframes are usually spread out every 5 seconds or less.
+                sc_val = int(stream.average_rate * 5)
+                tou = int(stream.time_base.denominator / stream.average_rate)
+
+            seek_cost.append(sc_val)
+            tous.append(tou)
+            decoders.append(cn.decode(stream))
+
+            if c == 0:
+                target_pix_fmt = stream.pix_fmt
 
     log.debug(f"Tous: {tous}")
     log.debug(f"Clips: {timeline.v}")
 
-    target_pix_fmt = pix_fmts[0] if pix_fmts[0] in allowed_pix_fmt else "yuv420p"
-    my_codec = get_vcodec(args.video_codec, inp, ctr)
+    target_pix_fmt = target_pix_fmt if target_pix_fmt in allowed_pix_fmt else "yuv420p"
+    log.debug(f"Target pix_fmt: {target_pix_fmt}")
 
     apply_video_later = True
 
-    if my_codec in encoders:
-        apply_video_later = encoders[my_codec]["pix_fmt"].isdisjoint(allowed_pix_fmt)
+    if args.video_codec in encoders:
+        apply_video_later = encoders[args.video_codec]["pix_fmt"].isdisjoint(
+            allowed_pix_fmt
+        )
 
     if args.scale != 1:
         apply_video_later = False
@@ -183,7 +205,7 @@ def render_av(
         "-s",
         f"{width}*{height}",
         "-framerate",
-        f"{timeline.fps}",
+        f"{timeline.timebase}",
         "-i",
         "-",
         "-pix_fmt",
@@ -195,25 +217,23 @@ def render_av(
     else:
         cmd = video_quality(cmd, args, inp, ctr)
 
+    # Setting SAR requires re-encoding so we do it here.
+    if timeline.inputs and timeline.inputs[0].videos:
+        if (sar := timeline.inputs[0].videos[0].sar) is not None:
+            cmd.extend(["-vf", f"setsar={sar.replace(':', '/')}"])
+
     cmd.append(spedup)
 
-    process2 = ffmpeg.Popen(
-        cmd, stdin=subprocess.PIPE, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
-    )
+    process2 = ffmpeg.Popen(cmd, stdin=PIPE, stdout=DEVNULL, stderr=DEVNULL)
     assert process2.stdin is not None
 
+    # First few frames can have an abnormal keyframe count, so never seek there.
     seek = 10
+
     seek_frame = None
     frames_saved = 0
 
-    # Keyframes are usually spread out every 5 seconds or less.
-    if args.no_seek:
-        SEEK_COST = 4294967295
-    else:
-        SEEK_COST = int(cns[0].streams.video[0].average_rate * 5)
-    SEEK_RETRY = SEEK_COST // 2
-
-    progress.start(timeline.end, "Creating new video")
+    bar.start(timeline.end, "Creating new video")
 
     null_img = Image.new("RGB", (width, height), args.background)
     null_frame = av.VideoFrame.from_image(null_img).reformat(format=target_pix_fmt)
@@ -222,7 +242,7 @@ def render_av(
     try:
         for index in range(timeline.end):
             # Add objects to obj_list
-            obj_list: List[Union[VideoFrame, Visual]] = []
+            obj_list: list[VideoFrame | Visual] = []
             for layer in timeline.v:
                 for lobj in layer:
                     if isinstance(lobj, VideoObj):
@@ -243,26 +263,33 @@ def render_av(
             frame = null_frame
             for obj in obj_list:
                 if isinstance(obj, VideoFrame):
+                    my_stream = cns[obj.src].streams.video[0]
                     if frame_index > obj.index:
-                        log.debug(f"Seek: {frame_index} -> beginning")
+                        log.debug(f"Seek: {frame_index} -> 0")
                         cns[obj.src].seek(0)
-                        frame = next(decoders[obj.src])
-                        frame_index = round(frame.time * timeline.fps)
+                        try:
+                            frame = next(decoders[obj.src])
+                            frame_index = round(frame.time * timeline.timebase)
+                        except StopIteration:
+                            pass
 
                     while frame_index < obj.index:
                         # Check if skipping ahead is worth it.
-                        if obj.index - frame_index > SEEK_COST and frame_index > seek:
-                            seek = frame_index + SEEK_RETRY
+                        if (
+                            obj.index - frame_index > seek_cost[obj.src]
+                            and frame_index > seek
+                        ):
+                            seek = frame_index + (seek_cost[obj.src] // 2)
                             seek_frame = frame_index
                             log.debug(f"Seek: {frame_index} -> {obj.index}")
                             cns[obj.src].seek(
                                 obj.index * tous[obj.src],
-                                stream=cns[obj.src].streams.video[0],
+                                stream=my_stream,
                             )
 
                         try:
                             frame = next(decoders[obj.src])
-                            frame_index = round(frame.time * timeline.fps)
+                            frame_index = round(frame.time * timeline.timebase)
                         except StopIteration:
                             log.debug(f"No source frame at {index=}. Using null frame")
                             frame = null_frame
@@ -334,16 +361,18 @@ def render_av(
                     img = Image.alpha_composite(img, obj_img)
                     frame = frame.from_image(img).reformat(format=target_pix_fmt)
 
+            if frame.format.name != target_pix_fmt:
+                frame = frame.reformat(format=target_pix_fmt)
             process2.stdin.write(frame.to_ndarray().tobytes())
 
             if index % 3 == 0:
-                progress.tick(index)
+                bar.tick(index)
 
-        progress.end()
+        bar.end()
         process2.stdin.close()
         process2.wait()
     except (OSError, BrokenPipeError):
-        progress.end()
+        bar.end()
         ffmpeg.run_check_errors(cmd, log, True)
         log.error("FFmpeg Error!")
 
