@@ -1,15 +1,16 @@
+from __future__ import annotations
+
 import os
 import sys
 import tempfile
 from dataclasses import dataclass, field
-from typing import List, Optional
 
 import numpy as np
 from numpy.typing import NDArray
 
 from auto_editor.ffwrapper import FFmpeg, FileInfo
+from auto_editor.utils.bar import Bar
 from auto_editor.utils.log import Log
-from auto_editor.utils.progressbar import ProgressBar
 from auto_editor.vanparse import ArgumentParser
 
 
@@ -17,10 +18,10 @@ from auto_editor.vanparse import ArgumentParser
 class LevelArgs:
     kind: str = "audio"
     track: int = 0
-    ffmpeg_location: Optional[str] = None
+    ffmpeg_location: str | None = None
     my_ffmpeg: bool = False
     help: bool = False
-    input: List[str] = field(default_factory=list)
+    input: list[str] = field(default_factory=list)
 
 
 def levels_options(parser: ArgumentParser) -> ArgumentParser:
@@ -63,12 +64,12 @@ def main(sys_args=sys.argv[1:]) -> None:
 
     ffmpeg = FFmpeg(args.ffmpeg_location, args.my_ffmpeg, False)
 
-    progress = ProgressBar("none")
+    bar = Bar("none")
     temp = tempfile.mkdtemp()
     log = Log(temp=temp)
 
     inp = FileInfo(args.input[0], ffmpeg, log)
-    fps = inp.get_fps()
+    timebase = inp.get_fps()
 
     if args.kind == "audio":
         from auto_editor.analyze.audio import audio_detection
@@ -86,25 +87,23 @@ def main(sys_args=sys.argv[1:]) -> None:
         if not os.path.isfile(read_track):
             log.error("Audio track file not found!")
 
-        sample_rate, audio_samples = read(read_track)
+        sr, samples = read(read_track)
+        print_float_list(audio_detection(samples, sr, timebase, bar, log))
 
-        print_float_list(audio_detection(audio_samples, sample_rate, fps, progress))
+    if args.kind in ("motion", "pixeldiff") and args.track >= len(inp.videos):
+        log.error(f"Video stream '{args.track}' does not exist.")
 
     if args.kind == "motion":
-        if args.track >= len(inp.videos):
-            log.error(f"Video track '{args.track}' does not exist.")
-
         from auto_editor.analyze.motion import motion_detection
 
-        print_float_list(motion_detection(inp.path, fps, progress, width=400, blur=9))
+        print_float_list(
+            motion_detection(inp.path, args.track, timebase, bar, width=400, blur=9)
+        )
 
     if args.kind == "pixeldiff":
-        if args.track >= len(inp.videos):
-            log.error(f"Video track '{args.track}' does not exist.")
-
         from auto_editor.analyze.pixeldiff import pixel_difference
 
-        print_int_list(pixel_difference(inp.path, fps, progress))
+        print_int_list(pixel_difference(inp.path, args.track, timebase, bar))
 
     log.cleanup()
 
