@@ -1,4 +1,5 @@
 # type: ignore
+from __future__ import annotations
 
 import logging
 import os
@@ -8,7 +9,8 @@ import subprocess
 import sys
 from dataclasses import dataclass, field
 from time import perf_counter
-from typing import Callable, List, NoReturn, Optional, Tuple
+from typing import Callable, NoReturn
+from fractions import Fraction
 
 import av
 import numpy as np
@@ -20,7 +22,7 @@ av.logging.set_level(av.logging.PANIC)
 
 @dataclass
 class TestArgs:
-    only: List[str] = field(default_factory=list)
+    only: list[str] = field(default_factory=list)
     help: bool = False
     category: str = "cli"
 
@@ -36,7 +38,7 @@ def test_options(parser):
     return parser
 
 
-def pipe_to_console(cmd: List[str]) -> Tuple[int, str, str]:
+def pipe_to_console(cmd: list[str]) -> tuple[int, str, str]:
     process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     stdout, stderr = process.communicate()
     return process.returncode, stdout.decode("utf-8"), stderr.decode("utf-8")
@@ -61,13 +63,13 @@ def clean_all() -> None:
     cleanup(os.getcwd())
 
 
-def get_runner() -> List[str]:
+def get_runner() -> list[str]:
     if platform.system() == "Windows":
         return ["py", "-m", "auto_editor"]
     return ["python3", "-m", "auto_editor"]
 
 
-def run_program(cmd: List[str]) -> None:
+def run_program(cmd: list[str]) -> None:
     no_open = "." in cmd[0]
     cmd = get_runner() + cmd
 
@@ -79,7 +81,7 @@ def run_program(cmd: List[str]) -> None:
         raise Exception(f"{stdout}\n{stderr}\n")
 
 
-def check_for_error(cmd: List[str], match=None) -> None:
+def check_for_error(cmd: list[str], match=None) -> None:
     returncode, stdout, stderr = pipe_to_console(get_runner() + cmd)
     if returncode > 0:
         if "Error!" in stderr:
@@ -160,7 +162,7 @@ class Tester:
         sys.exit(0)
 
 
-def main(sys_args: Optional[List[str]] = None):
+def main(sys_args: list[str] | None = None):
     if sys_args is None:
         sys_args = sys.argv[1:]
 
@@ -271,7 +273,7 @@ def main(sys_args: Optional[List[str]] = None):
         run_program(["example.mp4", "--margin", "3_0"])
         run_program(["example.mp4", "--sample_rate", "44100 Hz"])
         run_program(["example.mp4", "--sample_rate", "44.1 kHz"])
-        run_program(["example.mp4", "--silent_threshold", "4%"])
+        run_program(["example.mp4", "--edit", "audio:threshold=4%"])
 
     def backwards_range_test():
         """
@@ -351,7 +353,7 @@ def main(sys_args: Optional[List[str]] = None):
         run_program(["example.mp4", "--progress", "ascii"])
 
     def silent_threshold():
-        run_program(["resources/new-commentary.mp3", "--silent_threshold", "0.1"])
+        run_program(["resources/new-commentary.mp3", "--edit", "audio:threshold=0.1"])
 
     def track_tests():
         run_program(["resources/multi-track.mov", "--keep_tracks_seperate"])
@@ -382,6 +384,23 @@ def main(sys_args: Optional[List[str]] = None):
             assert cn.streams.video[0].height == 380
             assert cn.streams.audio[0].rate == 48000
 
+    def obj_makes_video():
+        run_program(
+            [
+                "resources/new-commentary.mp3",
+                "--add-rectangle",
+                "0,30,0,0,300,300,fill=blue",
+                "-o",
+                "out.mp4",
+            ]
+        )
+        with av.open("out.mp4") as cn:
+            assert len(cn.streams.video) == 1
+            assert len(cn.streams.audio) == 1
+            assert cn.streams.video[0].width == 1920
+            assert cn.streams.video[0].height == 1080
+            assert cn.streams.video[0].average_rate == 30
+
     def various_errors_test():
         check_for_error(
             ["example.mp4", "--add_rectangle", "0,60", "--cut_out", "60,end"]
@@ -628,6 +647,15 @@ def main(sys_args: Optional[List[str]] = None):
             "Logic operator must be between two editing methods",
         )
 
+    def yuv442p():
+        run_program(["resources/test_yuv422p.mp4"])
+
+    #  Issue 280
+    def SAR():
+        run_program(["resources/SAR-2by3.mp4"])
+        with av.open("resources/SAR-2by3_ALTERED.mp4") as cn:
+            assert cn.streams.video[0].sample_aspect_ratio == Fraction(2, 3)
+
     tests = []
 
     if args.category in ("unit", "all"):
@@ -642,6 +670,9 @@ def main(sys_args: Optional[List[str]] = None):
     if args.category in ("cli", "all"):
         tests.extend(
             [
+                SAR,
+                yuv442p,
+                obj_makes_video,
                 edit_positive_tests,
                 edit_negative_tests,
                 multi_track_edit,
