@@ -1,32 +1,21 @@
-from typing import List, Tuple, Union, overload
+from __future__ import annotations
+
+from fractions import Fraction
+from typing import overload
 
 import numpy as np
 from numpy.typing import NDArray
 
 from auto_editor.utils.log import Log
-from auto_editor.utils.types import Chunks, time
+from auto_editor.utils.types import time
 
 """
 To prevent duplicate code being pasted between scripts, common functions should be
 put here. Every function should be pure with no side effects.
 """
 
-# Turn long silent/loud array to formatted chunk list.
-# Example: [1, 1, 1, 2, 2] => [(0, 3, 1), (3, 5, 2)]
-def chunkify(arr: Union[np.ndarray, List[int]]) -> Chunks:
-    arr_length = len(arr)
 
-    chunks = []
-    start = 0
-    for j in range(1, arr_length):
-        if arr[j] != arr[j - 1]:
-            chunks.append((start, j, arr[j - 1]))
-            start = j
-    chunks.append((start, arr_length, arr[j]))
-    return chunks
-
-
-def to_timecode(secs: float, fmt: str) -> str:
+def to_timecode(secs: float | Fraction, fmt: str) -> str:
     sign = ""
     if secs < 0:
         sign = "-"
@@ -77,9 +66,9 @@ def remove_small(
 @overload
 def set_range(
     arr: NDArray[np.float_],
-    range_syntax: List[List[str]],
-    fps: float,
-    with_: int,
+    range_syntax: list[list[str]],
+    tb: Fraction,
+    with_: float,
     log: Log,
 ) -> NDArray[np.float_]:
     pass
@@ -88,16 +77,16 @@ def set_range(
 @overload
 def set_range(
     arr: NDArray[np.bool_],
-    range_syntax: List[List[str]],
-    fps: float,
-    with_: int,
+    range_syntax: list[list[str]],
+    tb: Fraction,
+    with_: float,
     log: Log,
 ) -> NDArray[np.bool_]:
     pass
 
 
-def set_range(arr, range_syntax, fps, with_, log):
-    def replace_variables_to_values(val: str, fps: float, log: Log) -> int:
+def set_range(arr, range_syntax, tb, with_, log):
+    def replace_variables_to_values(val: str, tb: Fraction, log: Log) -> int:
         if val == "start":
             return 0
         if val == "end":
@@ -106,16 +95,15 @@ def set_range(arr, range_syntax, fps, with_, log):
         try:
             value = time(val)
         except TypeError as e:
-            log.error(f"{e}")
+            log.error(e)
         if isinstance(value, int):
             return value
-        return round(float(value) * fps)
+        return round(float(value) * tb)
 
     for _range in range_syntax:
-        print(_range)
         pair = []
         for val in _range:
-            num = replace_variables_to_values(val, fps, log)
+            num = replace_variables_to_values(val, tb, log)
             if num < 0:
                 num += len(arr)
             pair.append(num)
@@ -123,12 +111,6 @@ def set_range(arr, range_syntax, fps, with_, log):
     return arr
 
 
-def seconds_to_frames(value: Union[int, str], fps: float) -> int:
-    if isinstance(value, str):
-        return int(float(value) * fps)
-    return value
-
-
 def cook(has_loud: NDArray[np.bool_], min_clip: int, min_cut: int) -> NDArray[np.bool_]:
     has_loud = remove_small(has_loud, min_clip, replace=1, with_=0)
     has_loud = remove_small(has_loud, min_cut, replace=0, with_=1)
@@ -167,31 +149,6 @@ def apply_margin(
     return has_loud
 
 
-def apply_mark_as(
-    has_loud: NDArray[np.bool_], has_loud_length: int, fps: float, args, log: Log
-) -> NDArray[np.bool_]:
-
-    if args.mark_as_loud != []:
-        has_loud = set_range(has_loud, args.mark_as_loud, fps, args.video_speed, log)
-
-    if args.mark_as_silent != []:
-        has_loud = set_range(has_loud, args.mark_as_silent, fps, args.silent_speed, log)
-    return has_loud
-
-
-def to_speed_list(
-    has_loud: NDArray[np.bool_], video_speed: float, silent_speed: float
-) -> NDArray[np.float_]:
-
-    speed_list = has_loud.astype(float)
-
-    # WARN: This breaks if speed is allowed to be 0
-    speed_list[speed_list == 1] = video_speed
-    speed_list[speed_list == 0] = silent_speed
-
-    return speed_list
-
-
 def merge(start_list: np.ndarray, end_list: np.ndarray) -> NDArray[np.bool_]:
     result = np.zeros((len(start_list)), dtype=np.bool_)
 
@@ -203,67 +160,16 @@ def merge(start_list: np.ndarray, end_list: np.ndarray) -> NDArray[np.bool_]:
     return result
 
 
-def parse_dataclass(unsplit_arguments, dataclass, log):
-    from dataclasses import fields
-
-    # Positional Arguments
-    #    --rectangle 0,end,10,20,20,30,#000, ...
-    # Keyword Arguments
-    #    --rectangle start=0,end=end,x1=10, ...
-
-    ARG_SEP = ","
-    KEYWORD_SEP = "="
-
-    d_name = dataclass.__name__
-
-    keys = [field.name for field in fields(dataclass)]
-    kwargs = {}
-    args = []
-
-    allow_positional_args = True
-
-    if unsplit_arguments == "":
-        return dataclass()
-
-    for i, arg in enumerate(unsplit_arguments.split(ARG_SEP)):
-        if i + 1 > len(keys):
-            log.error(f"{d_name} has too many arguments, starting with '{arg}'.")
-
-        if KEYWORD_SEP in arg:
-            allow_positional_args = False
-
-            parameters = arg.split(KEYWORD_SEP)
-            if len(parameters) > 2:
-                log.error(f"{d_name} invalid syntax: '{arg}'.")
-            key, val = parameters
-            if key not in keys:
-                log.error(f"{d_name} got an unexpected keyword '{key}'")
-
-            kwargs[key] = val
-        elif allow_positional_args:
-            args.append(arg)
-        else:
-            log.error(f"{d_name} positional argument follows keyword argument.")
-
-    try:
-        dataclass_instance = dataclass(*args, **kwargs)
-    except TypeError as err:
-        err_list = [d_name] + str(err).split(" ")[1:]
-        log.error(" ".join(err_list))
-
-    return dataclass_instance
-
-
-def get_stdout(cmd: List[str]) -> str:
+def get_stdout(cmd: list[str]) -> str:
     from subprocess import PIPE, STDOUT, Popen
 
     stdout, _ = Popen(cmd, stdout=PIPE, stderr=STDOUT).communicate()
     return stdout.decode("utf-8", "replace")
 
 
-def aspect_ratio(width: int, height: int) -> Union[Tuple[int, int], Tuple[None, None]]:
+def aspect_ratio(width: int, height: int) -> tuple[int, int]:
     if height == 0:
-        return None, None
+        return (0, 0)
 
     def gcd(a: int, b: int) -> int:
         while b:
