@@ -1,27 +1,17 @@
+from __future__ import annotations
+
 import difflib
 import re
 import sys
 import textwrap
 from dataclasses import dataclass
 from shutil import get_terminal_size
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    List,
-    Literal,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    TypeVar,
-    Union,
-)
+from typing import Any, Callable, Iterator, Literal, Sequence, TypeVar, Union
 
 import auto_editor
 from auto_editor.utils.log import Log
 
-T = TypeVar("T", bound=Callable)
+T = TypeVar("T", bound=type)
 Nargs = Union[int, Literal["*"]]
 
 
@@ -30,7 +20,7 @@ class Required:
     names: Sequence[str]
     nargs: Nargs = "*"
     type: type = str
-    choices: Optional[Sequence[str]] = None
+    choices: Sequence[str] | None = None
     help: str = ""
     _type: str = "required"
 
@@ -41,7 +31,8 @@ class Options:
     nargs: Nargs = 1
     type: type = str
     flag: bool = False
-    choices: Optional[Sequence[str]] = None
+    choices: Sequence[str] | None = None
+    pool: bool = False
     help: str = ""
     _type: str = "option"
 
@@ -56,7 +47,7 @@ def indent(text: str, prefix: str) -> str:
     def predicate(line: str) -> str:
         return line.strip()
 
-    def prefixed_lines():
+    def prefixed_lines() -> Iterator[str]:
         for line in text.splitlines(True):
             yield (prefix + line if predicate(line) else line)
 
@@ -80,9 +71,7 @@ def out(text: str) -> None:
     print("\n".join(wrapped_lines))
 
 
-def print_program_help(
-    reqs: List[Required], args: List[Union[Options, OptionText]]
-) -> None:
+def print_program_help(reqs: list[Required], args: list[Options | OptionText]) -> None:
     text = ""
     for arg in args:
         if isinstance(arg, OptionText):
@@ -95,13 +84,13 @@ def print_program_help(
     out(text)
 
 
-def get_help_data() -> Dict[str, Dict[str, str]]:
+def get_help_data() -> dict[str, dict[str, str]]:
     import json
     import os.path
 
     dirpath = os.path.dirname(os.path.realpath(__file__))
 
-    with open(os.path.join(dirpath, "help.json"), "r") as fileobj:
+    with open(os.path.join(dirpath, "help.json")) as fileobj:
         data = json.load(fileobj)
 
     assert isinstance(data, dict)
@@ -113,53 +102,69 @@ def to_underscore(name: str) -> str:
     return name[:2] + name[2:].replace("-", "_")
 
 
-def to_key(op: Union[Options, Required]) -> str:
+def to_key(op: Options | Required) -> str:
     """Convert option name to arg key.  e.g. --hello-world -> hello_world"""
     return op.names[0][:2].replace("-", "") + op.names[0][2:].replace("-", "_")
 
 
 def print_option_help(program_name: str, ns_obj: T, option: Options) -> None:
-    text = "  " + ", ".join(option.names) + f"\n    {option.help}\n\n"
-
-    data = get_help_data()
-
-    if option.names[0] in data[program_name]:
-        text += indent(data[program_name][option.names[0]], "    ") + "\n\n"
+    text = f"  {', '.join(option.names)}\n\n"
 
+    bar_len = 11
     if option.flag:
         text += "    type: flag\n"
+        bar_len = 15
     else:
-        text += f"    type: {option.type.__name__}\n"
-
         if option.nargs != 1:
-            text += f"    nargs: {option.nargs}\n"
+            _add = f"    nargs: {option.nargs}\n"
+            bar_len = len(_add)
+            text += _add
 
-        default = getattr(ns_obj, to_key(option))
+        _add = f"    type: {option.type.__name__}\n"
+        bar_len = len(_add)
+        text += _add
+
+        default: str | None = None
+        try:
+            default = getattr(ns_obj, to_key(option))
+        except AttributeError:
+            pass
 
         if default is not None:
-            text += f"    default: {default}\n"
+            if isinstance(default, tuple):
+                _add = f"    default: {','.join(map(str, default))}\n"
+            else:
+                _add = f"    default: {default}\n"
+            bar_len = len(_add)
+            text += _add
 
         if option.choices is not None:
             text += "    choices: " + ", ".join(option.choices) + "\n"
 
+    text += f"    {'-' * (bar_len - 5)}\n\n    {option.help}\n\n"
+    data = get_help_data()
+
+    if option.names[0] in data[program_name]:
+        text += indent(data[program_name][option.names[0]], "    ") + "\n"
+
     out(text)
 
 
-def get_option(name: str, options: List[Options]) -> Optional[Options]:
+def get_option(name: str, options: list[Options]) -> Options | None:
     for option in options:
         if name in option.names or name in map(to_underscore, option.names):
             return option
     return None
 
 
-def parse_value(option: Union[Options, Required], val: Optional[str]) -> Any:
+def parse_value(option: Options | Required, val: str | None) -> Any:
     if val is None and option.nargs == 1:
         Log().error(f"{option.names[0]} needs argument.")
 
     try:
         value = option.type(val)
     except TypeError as e:
-        Log().error(str(e))
+        Log().error(e)
 
     if option.choices is not None and value not in option.choices:
         my_choices = ", ".join(option.choices)
@@ -174,9 +179,9 @@ def parse_value(option: Union[Options, Required], val: Optional[str]) -> Any:
 class ArgumentParser:
     def __init__(self, program_name: str) -> None:
         self.program_name = program_name
-        self.requireds: List[Required] = []
-        self.options: List[Options] = []
-        self.args: List[Union[Options, OptionText]] = []
+        self.requireds: list[Required] = []
+        self.options: list[Options] = []
+        self.args: list[Options | OptionText] = []
 
     def add_argument(self, *args: str, **kwargs) -> None:
         x = Options(args, **kwargs)
@@ -195,8 +200,8 @@ class ArgumentParser:
     def parse_args(
         self,
         ns_obj: T,
-        sys_args: List[str],
-        macros: Optional[List[Tuple[Set[str], List[str]]]] = None,
+        sys_args: list[str],
+        macros: list[tuple[set[str], list[str]]] | None = None,
     ) -> T:
         if len(sys_args) == 0:
             out(get_help_data()[self.program_name]["_"])
@@ -217,7 +222,7 @@ class ArgumentParser:
         del macros
 
         ns = ns_obj()
-        option_names: List[str] = []
+        option_names: list[str] = []
 
         program_name = self.program_name
         requireds = self.requireds
@@ -233,15 +238,16 @@ class ArgumentParser:
         args.append(builtin_help)
 
         # Figure out command line options changed by user.
-        used_options: List[Options] = []
+        used_options: list[Options] = []
 
-        req_list: List[str] = []
+        req_list: list[str] = []
         req_list_name = requireds[0].names[0]
         setting_req_list = requireds[0].nargs != 1
 
-        option_list: List[str] = []
-        oplist_name: Optional[str] = None
-        oplist_coerce = str
+        option_list: Any = []
+        op_key: str = ""
+        oplist_name: str | None = None
+        oplist_coerce: Callable[[str], str] = str
 
         i = 0
         while i < len(sys_args):
@@ -249,9 +255,10 @@ class ArgumentParser:
             option = get_option(arg, options)
 
             if option is None:
-                if oplist_name is not None:
+                if oplist_name == "pool":
+                    option_list.append((op_key, arg))
+                elif oplist_name is not None:
                     option_list.append(oplist_coerce(arg))
-
                 elif requireds and not arg.startswith("--"):
                     if requireds[0].nargs == 1:
                         ns.__setattr__(req_list_name, parse_value(requireds[0], arg))
@@ -273,13 +280,13 @@ class ArgumentParser:
                         )
                     Log().error(f"Unknown {label}: {arg}")
             else:
-                if oplist_name is not None:
+                if oplist_name is not None and oplist_name != "pool":
                     ns.__setattr__(oplist_name, option_list)
 
-                if option in used_options:
-                    Log().error(f"Cannot repeat option {option.names[0]} twice.")
-
-                used_options.append(option)
+                if not option.pool:
+                    if option in used_options:
+                        Log().error(f"Cannot repeat option {option.names[0]} twice.")
+                    used_options.append(option)
 
                 option_list = []
                 oplist_name = None
@@ -293,8 +300,13 @@ class ArgumentParser:
                     sys.exit()
 
                 if option.nargs != 1:
-                    oplist_name = key
-                    ns.__setattr__(key, parse_value(option, next_arg))
+                    if option.pool:
+                        oplist_name = "pool"
+                        op_key = key
+                        ns.pool.append((key, next_arg))
+                    else:
+                        oplist_name = key
+                        ns.__setattr__(oplist_name, parse_value(option, next_arg))
                 elif option.flag:
                     ns.__setattr__(key, True)
                 else:
@@ -302,7 +314,9 @@ class ArgumentParser:
                     i += 1
             i += 1
 
-        if oplist_name is not None:
+        if oplist_name == "pool":
+            ns.pool += option_list
+        elif oplist_name is not None:
             ns.__setattr__(oplist_name, option_list)
 
         if setting_req_list:
